@@ -20,6 +20,13 @@ pub mod _changelog_ {}
 
 pub mod abi;
 
+#[cfg(all(feature = "nightly", feature = "unstable"))]
+#[cfg_attr(
+    feature = "doc_cfg",
+    doc(cfg(all(feature = "nightly", feature = "unstable")))
+)]
+pub mod macros;
+
 macro_rules! unstable_module {
     {$(
         $( #[macro_use $($unused:tt)*] )*
@@ -55,7 +62,6 @@ unstable_module! {
     pub mod messagebuffer;
     pub mod mutex;
     pub mod prioritydataqueue;
-    #[cfg(any(feature = "fmp3", feature = "solid_fmp3", feature = "none"))]
     pub mod processor;
     pub mod semaphore;
     pub mod task;
